﻿using Styx.Common;
using Styx.CommonBot;
using Styx.Loaders;
using Styx.TreeSharp;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Windows.Media;

namespace AutoArena
{
    public sealed class BotLoader : BotBase
    {
        private BotBase _bot;

        private const string SvnUrl = "http://code.taobao.org/svn/AutoArena/trunk/";
        private const string VerUrl = "http://code.taobao.org/svn/AutoArena/trunk/Changelog.txt";
        private static readonly string BotPath = Utilities.AssemblyDirectory + @"\Bots\" + "AutoArena";
        private static string path = BotPath + @"\AutoArena.dll";

        private static readonly Regex _linkPattern = new Regex(@"<li><a href="".+"">(?<ln>.+(?:..))</a></li>",
                                                               RegexOptions.CultureInvariant);

        public BotLoader()
        {


            if (TreeRoot.IsRunning)
            {
                LoggingWarning("运行中，请先停止！");
            }
            else
            {
                CheckForUpdate();

                LoggingInfo("Load Bot: " + path);

                try
                {
                    byte[] Bytes = File.ReadAllBytes(path);
                    Assembly asm = Assembly.Load(Bytes);

                    foreach (Type t in asm.GetTypes())
                    {
                        if (t.IsSubclassOf(typeof(BotBase)) && t.IsClass)
                        {
                            object obj = Activator.CreateInstance(t);
                            _bot = (BotBase)obj;
                        }
                    }
                }
                catch (Exception error)
                {
                    LoggingError(error.Message);
					LoggingInfo("版本加载失败.请登录 www.592.la 下载最新版本");
                }
            }
        }

        private static void LoggingError(string message)
        {
            message = "[BotLoader] " + message;

            Trace.WriteLine(message);
            Logging.Write(Colors.OrangeRed, message);
        }

        private static void LoggingInfo(string message)
        {
            message = "[BotLoader] " + message;

            Trace.WriteLine(message);
            Logging.Write(Colors.DarkGray, message);
        }

        private static void LoggingWarning(string message)
        {
            message = "[BotLoader] " + message;

            Trace.WriteLine(message);
            Logging.Write(Colors.DarkOrange, message);
        }

        #region Override Properties

        public override Form ConfigurationForm
        {
            get
            {
                return _bot.ConfigurationForm;
            }
        }

        public override bool IsPrimaryType
        {
            get { return _bot.IsPrimaryType; }
        }
        public override string Name
        {
            get { return _bot.Name; }
        }
        public override PulseFlags PulseFlags
        {
            get { return _bot.PulseFlags; }
        }
        public override bool RequirementsMet
        {
            get { return _bot.RequirementsMet; }
        }
        public override bool RequiresProfile
        {
            get { return _bot.RequiresProfile; }
        }
        public override bool RequiresProfileScope
        {
            get { return _bot.RequiresProfileScope; }
        }
        public override Composite Root
        {
            get { return _bot.Root; }
        }

        #endregion

        #region Override Methods

        public override void Initialize()
        {
            _bot.Initialize();
        }
        public override void OnDeselected()
        {
            _bot.OnDeselected();
        }
        public override void OnPaused()
        {
            _bot.OnPaused();
        }
        public override void OnResumed()
        {
            _bot.OnResumed();
        }
        public override void OnSelected()
        {
            _bot.OnSelected();
        }
        public override void Pulse()
        {
            _bot.Pulse();
        }
        public override void Start()
        {
            _bot.Start();
        }
        public override void Stop()
        {
            _bot.Stop();
        }

        #endregion

        #region 版本检测

        /// <summary>
        /// 获取文件的版本号
        /// </summary>
        /// <param name="filePath">文件的完整路径</param>
        /// <returns>文件的版本号</returns>
        private static string GetFileVersion(string filePath)
        {
            string FileVersions = "";

            try
            {
                FileVersionInfo file = FileVersionInfo.GetVersionInfo(filePath);
                FileVersions = file.ProductVersion;
            }
            catch (Exception)
            {
                FileVersions = "1.0.0";
            }

            return FileVersions;
        }

        public static void CheckForUpdate()
        {
            try
            {
                LoggingInfo("Checking for new version");
                Version remoteRev = GetRevision();
                if (Version.Parse(GetFileVersion(path)) < remoteRev)
                {
                    LoggingInfo(String.Format("A new version was found : {0}", remoteRev));
                    LoggingInfo("Downloading Update");
                    DownloadFilesFromSvn(new WebClient(), SvnUrl);
                    LoggingInfo("Download complete.");
                    //AutoArenaSettings.Instance.CurrentRevision = remoteRev;
                    //AutoArenaSettings.Instance.Save();
                }
                else
                {
                    LoggingInfo("No updates found");
                }
            }
            catch (Exception ex)
            {
                LoggingInfo(ex.ToString());
				LoggingInfo("版本更新失败.请登录 www.592.la 下载最新版本");
            }
        }

        private static Version GetRevision()
        {
            var client = new WebClient();
            string html = client.DownloadString(VerUrl).Split(new char[] { '\n' })[0].Split(new char[] { ':' })[1];
            return Version.Parse(html);
            throw new Exception("Unable to retreive revision");
        }

        private static void DownloadFilesFromSvn(WebClient client, string url)
        {
            string html = client.DownloadString(url);
            MatchCollection results = _linkPattern.Matches(html);

            IEnumerable<Match> matches = from match in results.OfType<Match>()
                                         where match.Success && match.Groups["ln"].Success
                                         select match;
            foreach (Match match in matches)
            {
                string file = RemoveXmlEscapes(match.Groups["ln"].Value);
                string newUrl = url + file;
                if (newUrl[newUrl.Length - 1] == '/') // it's a directory...
                {
                    DownloadFilesFromSvn(client, newUrl);
                }
                else // its a file.
                {
                    string filePath, dirPath;
                    if (url.Length > SvnUrl.Length)
                    {
                        string relativePath = url.Substring(SvnUrl.Length);
                        dirPath = Path.Combine(BotPath, relativePath);
                        filePath = Path.Combine(dirPath, file);
                    }
                    else
                    {
                        dirPath = Environment.CurrentDirectory;
                        filePath = Path.Combine(BotPath, file);
                    }
                    LoggingInfo(String.Format("Downloading {0}", file));
                    if (!Directory.Exists(dirPath))
                        Directory.CreateDirectory(dirPath);
                    client.DownloadFile(newUrl, filePath);
                }
            }
        }

        private static string RemoveXmlEscapes(string xml)
        {
            return
                xml.Replace("&amp;", "&").Replace("&lt;", "<").Replace("&gt;", ">").Replace("&quot;", "\"").Replace(
                    "&apos;", "'");
        }
        #endregion
    }
}
